@@ -40,3 +40,12 @@ intersphinx_mapping = {
 extlinks = {
     'issue': ('https://github.com/felix-hilden/sphinx-codeautolink/issues/%s', '#'),
 }
+
+
+def setup(app):
+    app.add_object_type(
+        'confval',
+        'confval',
+        objname='configuration value',
+        indextemplate='pair: %s; configuration value'
+    )
