@@ -1,15 +1,14 @@
 """Sphinx extension implementation."""
 from dataclasses import dataclass
-from functools import lru_cache
-from importlib import import_module
-from typing import Dict, List, Optional, Tuple, Any, Set
+from typing import Dict, List, Optional, Set
 from pathlib import Path
 
 from sphinx.ext.intersphinx import InventoryAdapter
 
 from .backref import CodeRefsVisitor, CodeExample
-from .block import CodeBlockAnalyser, SourceTransform, link_html, Name, NameBreak
+from .block import CodeBlockAnalyser, SourceTransform, link_html
 from .cache import DataCache
+from .resolve import resolve_location
 
 
 @dataclass
@@ -30,6 +29,7 @@ class SphinxCodeAutoLink:
         self.code_refs: Dict[str, List[CodeExample]] = {}
         self._inventory = {}
         self.outdated_docs: Set[str] = set()
+        self.global_preface: List[str] = []
 
     def build_inited(self, app):
         """Handle initial setup."""
@@ -37,7 +37,7 @@ class SphinxCodeAutoLink:
             self.do_nothing = True
             return
 
-        self.cache = DataCache(app.srcdir)
+        self.cache = DataCache(app.doctreedir, app.srcdir)
         self.cache.read()
         self.outdated_docs = {str(Path(d)) for d in app.builder.get_outdated_docs()}
 
@@ -46,20 +46,28 @@ class SphinxCodeAutoLink:
             str(Path(__file__).parent.with_name('static').absolute())
         )
 
+        preface = app.config.codeautolink_global_preface
+        if preface:
+            self.global_preface = preface.split('\n')
+
     def autodoc_process_docstring(self, app, what, name, obj, options, lines):
         """Handle autodoc-process-docstring event."""
         if self.do_nothing:
             return
 
         if app.config.codeautolink_autodoc_inject:
-            lines.append(f'.. code-refs:: {name}')
+            lines.append('')
+            lines.append('.. autolink-examples:: ' + name)
+            lines.append('   :collapse:')
 
     def parse_blocks(self, app, doctree):
         """Parse code blocks for later link substitution."""
         if self.do_nothing:
             return
 
-        visitor = CodeBlockAnalyser(doctree, source_dir=app.srcdir)
+        visitor = CodeBlockAnalyser(
+            doctree, source_dir=app.srcdir, global_preface=self.global_preface
+        )
         doctree.walkabout(visitor)
         self.cache.transforms[visitor.current_document] = visitor.source_transforms
 
@@ -157,67 +165,3 @@ def transpose_inventory(inv: dict, relative_to: str):
                 location = str(Path(location).relative_to(relative_to))
             transposed[item] = location
     return transposed
-
-
-def resolve_location(chain: Name) -> Optional[str]:
-    """Find the final type that a name refers to."""
-    comps = []
-    for comp in chain.import_components:
-        if comp == NameBreak.call:
-            new = locate_type(tuple(comps))
-            if new is None:
-                return
-            comps = new.split('.')
-        else:
-            comps.append(comp)
-    return '.'.join(comps)
-
-
-@lru_cache(maxsize=None)
-def locate_type(components: Tuple[str]) -> Optional[str]:
-    """Find type hint and resolve to new location."""
-    value, index = closest_module(components)
-    if index is None or index == len(components):
-        return
-    remaining = components[index:]
-    real_location = '.'.join(components[:index])
-    for i, component in enumerate(remaining):
-        value = getattr(value, component, None)
-        real_location += '.' + component
-        if value is None:
-            return
-
-        if isinstance(value, type):
-            # We don't differentiate between classmethods and ordinary methods,
-            # as we can't guarantee correct runtime behavior anyway.
-            real_location = fully_qualified_name(value)
-
-        # A possible function / method call needs to be last in the chain.
-        # Otherwise we might follow return types on function attribute access.
-        elif callable(value) and i == len(remaining) - 1:
-            ret_annotation = value.__annotations__.get('return', None)
-            if not ret_annotation or hasattr(ret_annotation, '__origin__'):
-                return
-            real_location = fully_qualified_name(ret_annotation)
-
-    return real_location
-
-
-def fully_qualified_name(type_: type) -> str:
-    """Construct the fully qualified name of a type."""
-    return getattr(type_, '__module__', '') + '.' + getattr(type_, '__qualname__', '')
-
-
-@lru_cache(maxsize=None)
-def closest_module(components: Tuple[str]) -> Tuple[Any, Optional[int]]:
-    """Find closest importable module."""
-    mod = None
-    for i in range(1, len(components) + 1):
-        try:
-            mod = import_module('.'.join(components[:i]))
-        except ImportError:
-            break
-    else:
-        return None, None
-
-    return mod, i - 1
