@@ -1,7 +1,7 @@
 """Code block processing."""
 import re
 
-from typing import List
+from typing import List, Union, Optional
 from pathlib import Path
 from warnings import warn
 from dataclasses import dataclass
@@ -96,7 +96,19 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         """Pop latest title."""
         self.title_stack.pop()
 
+    def visit_doctest_block(self, node):
+        """Visit a Python doctest block."""
+        return self.parse_source(node, 'pycon')
+
     def visit_literal_block(self, node: nodes.literal_block):
+        """Visit a generic literal block."""
+        return self.parse_source(node, node.get('language', None))
+
+    def parse_source(
+        self,
+        node: Union[nodes.literal_block, nodes.doctest_block],
+        language: Optional[str]
+    ):
         """Analyse Python code blocks."""
         prefaces = self.prefaces
         self.prefaces = []
@@ -108,7 +120,7 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         if (
             len(node.children) != 1
             or not isinstance(node.children[0], nodes.Text)
-            or node.get('language', None) not in ('py', 'python', 'pycon')
+            or language not in ('py', 'python', 'pycon')
         ):
             return
 
@@ -116,14 +128,13 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
             self.current_document, self.current_refid, list(self.title_stack)
         )
         source = node.children[0].astext()
-        transform = SourceTransform(source, [], example)
-        self.source_transforms.append(transform)
 
         if skip:
             return
 
-        if node['language'] == 'pycon':
+        if language == 'pycon':
             in_statement = False
+            source = re.sub(r'^\s*<BLANKLINE>', '', source, flags=re.MULTILINE)
             clean_lines = []
             for line in source.split('\n'):
                 if line.startswith('>>> '):
@@ -138,6 +149,9 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         else:
             clean_source = source
 
+        transform = SourceTransform(source, [], example)
+        self.source_transforms.append(transform)
+
         modified_source = '\n'.join(
             self.global_preface
             + self.concat_sources
@@ -166,20 +180,8 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         if self.concat_section or self.concat_global:
             self.concat_sources.extend(prefaces + [clean_source])
 
-        for name in names:
-            if name.lineno < 1:
-                continue  # From concatenated source
-
-            if name.lineno != name.end_lineno:
-                msg = (
-                    'sphinx-codeautolinks: multiline names are not supported, '
-                    f'found `{name.code_str}` in document {self.current_document} '
-                    f'on lines {name.lineno} - {name.end_lineno}'
-                )
-                warn(msg, RuntimeWarning)
-                continue
-
-            transform.names.append(name)
+        # Remove transforms from concatenated sources
+        transform.names.extend([n for n in names if n.lineno > 0])
 
 
 def link_html(
@@ -191,7 +193,9 @@ def link_html(
     blocks = (
         list(soup.find_all('div', attrs={'class': 'highlight-python notranslate'}))
         + list(soup.find_all('div', attrs={'class': 'highlight-pycon notranslate'}))
+        + list(soup.find_all('div', attrs={'class': 'doctest'}))
     )
+    blocks = sorted(blocks, key=lambda tag: tag.sourceline)
     inners = [block.select('div > pre')[0] for block in blocks]
 
     up_lvls = len(document.relative_to(out_dir).parents) - 1
@@ -200,23 +204,24 @@ def link_html(
         + '{link}" title="{title}" class="sphinx-codeautolink-a">{text}</a>'
     )
     name_pattern = '<span class="n">{name}</span>'
-    maybe_decor_pattern = '<span class="nd?">@?{name}</span>'
-    period = '<span class="o">.</span>'
+    # Pygments has special classes for decorators (nd) and builtins (nb)
+    first_item_pattern = '<span class="n[bd]?">@?{name}</span>'
+    period = r'\s*<span class="o">.</span>\s*'
 
     # Expression asserts no dots before or after content nor a link after,
     # i.e. a self-contained name or attribute that hasn't been linked yet
     # so we are free to replace any occurrences, since the order of
     # multiple identical replacements doesn't matter.
-    base_ex = r'(?<!<span class="o">\.</span>){content}(?!(<span class="o">\.)|(</a>))'
-    # Potentially instead assert an initial closing parenthesis followed by a dot
-    call_ex = (
-        r'(?<=\)</span><span class="o">\.</span>)'
-        r'{content}(?!(<span class="o">\.)|(</a>))'
-    )
+    end = r'(?!(<span class="o">\.)|(</a>))'
+    base_ex = r'(?<!<span class="o">\.</span>)(){content}' + end
+    # Potentially instead assert an initial closing parenthesis followed by a dot.
+    # We cannot use a lookbehind here because builtin re doesn't support it,
+    # so instead we use a match group to remove the non-content.
+    call_ex = r'(\)</span>\s*<span class="o">\.</span>\s*){content}' + end
 
     for trans in transforms:
         for ix in range(len(inners)):
-            if trans.source == ''.join(inners[ix].strings).rstrip():
+            if trans.source.rstrip() == ''.join(inners[ix].strings).rstrip():
                 inner = inners.pop(ix)
                 break
         else:
@@ -232,31 +237,35 @@ def link_html(
         for name in trans.names:
             parts = name.code_str.split('.')
             pattern = period.join(
-                [maybe_decor_pattern.format(name=parts[0])]
+                [first_item_pattern.format(name=parts[0])]
                 + [name_pattern.format(name=p) for p in parts[1:]]
             )
 
-            line = lines[name.lineno - 1]
+            begin_line = name.lineno - 1
+            end_line = name.end_lineno - 1
+            selection = '\n'.join(lines[begin_line:end_line + 1])
 
             # Reverse because a.b = a.b should replace from the right
             ex = call_ex if name.previous == NameBreak.call else base_ex
-            matches = list(re.finditer(ex.format(content=pattern), line))[::-1]
+            matches = list(re.finditer(ex.format(content=pattern), selection))[::-1]
             if not matches:
                 msg = (
                     f'Could not match transformation of `{name.code_str}` '
-                    f'on source line {name.lineno} in document "{document}", '
-                    f'source:\n{trans.source}'
+                    f'on source lines {name.lineno}-{name.end_lineno} '
+                    f'in document "{document}", source:\n{trans.source}'
                 )
                 warn(msg, RuntimeWarning)
                 continue
 
             start, end = matches[0].span()
+            start += len(matches[0].group(1))
             link = link_pattern.format(
                 link=inventory[name.resolved_location],
                 title=name.resolved_location,
-                text=line[start:end]
+                text=selection[start:end]
             )
-            lines[name.lineno - 1] = line[:start] + link + line[end:]
+            transformed = selection[:start] + link + selection[end:]
+            lines[begin_line:end_line + 1] = transformed.split('\n')
 
         inner.replace_with(BeautifulSoup('\n'.join(lines), 'html.parser'))
 
